#include "epix2.h"
using namespace ePiX2;

const double K(0.0125);

Point F(double x, double y, double z)
{
  return Point(1.05*x-0.2*y, 0.95*y+0.2*x, 1.2*z);
  // return Point(x*(1+2*K*y), y - K*(x-y)*(x+y), 1.25*z+2*K*(x-y));
}

const double MAX(10);

int main() {
  World world;
  Camera camera;

  Picture picture(Pair(-MAX,-MAX), Pair(MAX,MAX), "6x6in");

#ifdef COLOR
  picture.backing(Black());
  Duo_Pen paint(bold(Red(1.6)), Pen(Red(0.6), "2pt"));
#else
  Duo_Pen paint(plain(Black()), Pen(White(), "2pt"));
#endif

  const int N(5);

  for (int i=-N; i<=N; ++i)
    for (int j=-N; j<=N; ++j)
      for (int k=-N; k<=N; ++k)
	world << Segment(Point(i, j, k), F(i, j, k)).stroke(paint);

  camera.at(Point(5, 4, 3));
  camera.range(20);

#ifdef COLOR
  world.fog(Black(), 0.05, 10);
#else
  world.fog(White(), 0.1, 15);
#endif

  world.photo(picture, camera);
  picture.crop();
  picture.print_to("vfield.eepic", PSTricks());
}
