/* 
 *  Basis.h -- epix2::Basis class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 01, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * A Basis is guaranteed to be orthonormal.  
 * This file provides:
 *  - the Basis class
 *  - translation, rotation, and reflection operators
 */

#ifndef EPIX2_BASIS
#define EPIX2_BASIS

#include "Point.h"
#include "Vector.h"

namespace ePiX2 {

  /* * * Basis.h * * */

  class Basis {
  public:
    Basis(void); // standard basis at the origin

    // Gram-Schmidt {arg1, arg2, arg3} -> {sea, sky, eye}, with arg3 || eye
    Basis(const Point&, const Vector&, const Vector&, const Vector&);

    // get data
    Point here(void) const { return m_O; }
    Vector sea(void) const { return e1; }
    Vector sky(void) const { return e2; }
    Vector eye(void) const { return e3; }

    // coordinates of arg with respect to us
    Point coords(const Point& arg, const double scale=1) const;
    // inverse coordinates of arg with respect to us
    Point anti_coords(const Point& arg, const double scale=1) const;

    // transform
    Basis& move_to(const Point&);
    Basis& rotate_sea(const double angle);
    Basis& rotate_sky(const double angle);
    Basis& rotate_eye(const double angle);

    Basis& rotate(const double angle, const Vector& axis=E_3());
    Basis& reflect(const Vector& axis=E_3());

  private:
    Point  m_O; // location
    // orthonormal triple
    Vector e1;
    Vector e2;
    Vector e3;
  }; // end of class Basis

  bool operator== (const Basis&, const Basis&);

} /* end of namespace */

#endif /* EPIX2_BASIS */
