/*
 * Color_CMYK.h -- ePiX2::CMYK_Color class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 14, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef EPIX2_COLOR_CMYK
#define EPIX2_COLOR_CMYK
#include <string>

#include "Color.h"

class Format;

namespace ePiX2 {

  class CMYK_Color : public Color_Base {
  public:
    CMYK_Color(const double c=1, const double m=1, const double y=1,
	       const double k=1);
    CMYK_Color(const RGB_Densities&);

    CMYK_Color* clone() const;

    Color filter(const Color_Base&) const;

    CMYK_Color& operator*= (const double c);

    CMYK_Color& blend(const Color_Base&, const double);
    CMYK_Color& superpose(const Color_Base&);
    CMYK_Color& invert();

    std::string name() const;
    std::string print_to(const Format&) const;

    RGB_Densities to_rgb() const;

    // These need to call our private constructor
    friend Color CyanK(const double);
    friend Color MagentaK(const double);
    friend Color YellowK(const double);
    friend Color CMYK_White(const double);
    friend Color CMYK_Black(const double);
    friend Color CMYK_Neutral();

  private:
    double cyan;
    double magenta;
    double yellow;
    double black;

    // Construct a CMYK color without checking arguments. Final argument
    // only changes call signature
    CMYK_Color(const double c, const double m, const double y,
	       const double k, const bool);
  }; // end of class CMYK_Color

} // end of namespace
#endif /* EPIX2_COLOR_CMYK */
