/*
 * Color_Gray.cc -- ePiX2::Gray_Color functions
 *
 * This file is part of ePiX, a program for creating high-quality
 * figures in LaTeX
 *
 * Version 2.0pre
 *
 * Last Change: August 27, 2006
 *
 *
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifdef DEBUG
#include "Debug.h"
#endif

#include <sstream>
#include <string>

#include <cmath>

#include "Functions.h"

#include "Output.h"

#include "Color_Utils.h"
#include "Color.h"
#include "Color_Gray.h"

namespace ePiX2 {

  Gray_Color::Gray_Color(const double d)
    : m_gray(clip_0_1(d)) { }


  Gray_Color::Gray_Color(const RGB_Densities& s)
    : m_gray((1.0/3)*(s.m_dens_red+s.m_dens_green+s.m_dens_blue)) { }

  Color_Base::RGB_Densities Gray_Color::to_rgb() const
  {
    return Color_Base::RGB_Densities(m_gray, m_gray, m_gray);
  }

  Gray_Color* Gray_Color::clone() const
  {
    return new Gray_Color(*this);
  }


  Color Gray_Color::filter(const Color_Base& color) const
  {
    Color_Base::RGB_Densities s(color.to_rgb());

    return Gray(min(m_gray,
		    (1.0/3)*(s.m_dens_red+s.m_dens_green+s.m_dens_blue)));
  }

  Gray_Color& Gray_Color::operator*= (const double c)
  {
    m_gray = clip_0_1(c*m_gray);
    return *this;
  }

  Gray_Color& Gray_Color::blend(const Color_Base& color, const double d)
  {
    Color_Base::RGB_Densities s(color.to_rgb());
    double wt(clip_0_1(d));

    m_gray += wt*((1.0/3)*(s.m_dens_red+s.m_dens_green+s.m_dens_blue)-m_gray);

    return *this;
  }

  Gray_Color& Gray_Color::superpose(const Color_Base& color)
  {
    Color_Base::RGB_Densities s(color.to_rgb());

    m_gray = clip_0_1((1.0/3)*(s.m_dens_red   +
			       s.m_dens_green +
			       s.m_dens_blue) + m_gray);

    return *this;
  }

  Gray_Color& Gray_Color::invert()
  {
    m_gray = 1 - m_gray;
    return *this;
  }

  // string identifier -- not output-specific
  std::string Gray_Color::name() const
  {
    std::ostringstream nm;

    nm << "gray_" << dtohex(m_gray);

    return nm.str();
  }

  std::string Gray_Color::print_to(const Format& fmt) const
  {
    // rd = "round off"; treat like rgb
    return fmt.print_color("rgb", name(), rd(m_gray), rd(m_gray), rd(m_gray));
  }
} /* end of namespace */
