/*
 *  Color_RGB.h -- epix2::RGB_Color class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 14, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef EPIX2_COLOR_RGB
#define EPIX2_COLOR_RGB
#include <string>

#include "Color.h"

class Format;

namespace ePiX2 {
  class RGB_Color : public Color_Base {
  public:
    RGB_Color(const double r=0, const double g=0, const double b=0);
    RGB_Color(const RGB_Densities&);

    RGB_Color* clone() const;

    Color filter(const Color_Base&) const;

    RGB_Color& operator*= (const double c);

    RGB_Color& blend(const Color_Base&, const double);
    RGB_Color& superpose(const Color_Base&);
    RGB_Color& invert();

    std::string name() const;
    std::string print_to(const Format&) const;

    RGB_Densities to_rgb() const;

  private:
    double red;
    double green;
    double blue;
  }; // end of class RGB_Color

} // end of namespace
#endif /* EPIX2_COLOR_RGB */
