/*
 * Constants.h -- ePiX's enums and internal constants
 *
 * This file is part of ePiX, a preprocessor for creating high-quality
 * line figures in LaTeX
 *
 * Version 2.0pre
 * Last Change: August 04, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef EPIX2_CONSTANTS
#define EPIX2_CONSTANTS

#include "Length.h"

namespace ePiX2 {

  // ePiX's idea of too small/large (10^\pm 5)
  const double EPIX2_EPSILON(0.00001);
  const double EPIX2_INFTY(100000.0);

  // output file maximum width
  const unsigned int FILE_WIDTH(78);

  // default line widths
  const Length PLAIN_WIDTH("0.6pt");
  const Length BOLD_WIDTH("1.0pt");
  const Length BBOLD_WIDTH("1.5pt");
  const Length LABEL_PAD("3pt");     // PSTricks default


  // Label alignment: B = baseline, b = bottom
  enum align_type 
    {B, Br, Bl, c, r, tr, rt, t, tl, lt, l, bl, lb, b, br, rb};

  enum mark_type {MK_NONE, MK_CIRC, MK_SPOT, MK_RING,
		  MK_DOT, MK_DDOT, MK_PLUS, MK_OPLUS,
		  MK_TIMES, MK_OTIMES, MK_DIAMOND,
		  MK_UP, MK_DOWN, MK_BOX, MK_BBOX,
		  MK_HTICK, MK_VTICK};

  enum font_type { ROMAN, ITALIC, BFACE, TTYPE };

  enum photo_type { PHOTO, FLASH, X_RAY, TRACE };

} /* end of namespace */

#endif /* EPIX2_CONSTANTS */
