/* 
 *  Edge.h -- ePiX2 Edge data classes, designed not to be seen by the user
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: July 09, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef EPIX2_EDGE
#define EPIX2_EDGE

#include "Point.h"
#include "Pair.h"

#include "Styles.h"

namespace ePiX2 {

  class Edge3d {
  public:
    Edge3d(const Point& fm, const Point& to, bool seen)
      : m_tail(fm), m_head(to), m_seen(seen) { }

    Point tail(void) const { return m_tail; }
    Point head(void) const { return m_head; }
    bool  seen(void) const { return m_seen; }

  private:
    Point m_tail, m_head;
    bool m_seen;
  }; // end of class Edge3d


  class Edge2d {
  public:
    Edge2d(const Pair& fm, const Pair& to, bool seen)
      : m_tail(fm), m_head(to), m_seen(seen) { }

    Pair tail(void) const { return m_tail; }
    Pair head(void) const { return m_head; }
    bool seen(void) const { return m_seen; }

  private:
    Pair m_tail, m_head;
    bool m_seen;
  }; // end of class Edge2d


  class Wire3d {
    friend class Plate3d;
    friend class Outline3d;

  public:
    Wire3d(const Edge3d& data, const Pen& line, const Pen& base)
      : m_data(data), m_line(line), m_base(base) { }

    Wire3d(const Point& tail, const Point& head,
	   const Pen& line, const Pen& base)
      : m_data(Edge3d(tail, head, true)), m_line(line), m_base(base) { }

    Wire3d(const Point& tail, const Point& head, const Pen& line)
      : m_data(Edge3d(tail, head, true)), m_line(line), m_base(line) { }

  private:
    Edge3d m_data;

    Pen m_line;
    Pen m_base;
  }; // end of class Wire3d


  class Wire2d {
    friend class Plate2d;
    friend class Outline2d;

  public:
    Wire2d(const Edge2d& data, const Pen2d& line, const Pen2d& base)
      : m_data(data), m_line(line), m_base(base) { }

    Wire2d(const Pair& tail, const Pair& head,
	   const Pen2d& line, const Pen2d& base)
      : m_data(Edge2d(tail, head, true)), m_line(line), m_base(base) { }

    Wire2d(const Pair& tail, const Pair& head, const Pen2d& line)
      : m_data(Edge2d(tail, head, true)), m_line(line), m_base(line) { }

  private:
    Edge2d m_data;

    Pen2d  m_line;
    Pen2d  m_base;
  }; // end of class Wire2d

} /* end of namespace */

#endif /* EPIX2_EDGE */
