/*
 * Fog.cc -- ePiX2::Fog class
 *
 * This file is part of ePiX, a program for creating high-quality
 * figures in LaTeX
 *
 * Version 2.0pre
 *
 * Last Change: August 14, 2006
 *
 *
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifdef DEBUG
#include "Debug.h"
#endif

#include <cmath>

#include "Functions.h"
#include "Color.h"
#include "Fog.h"

namespace ePiX2 {

  Fog::Fog(const Color& col, const double dens, const double clear)
      : m_color(col), m_dens(dens), m_clear(clear) { }

  void Fog::clear_to(const double dist)
  {
    m_clear = dist;
  }

  Color Fog::mutes(const Color& col, const double d) const
  {
    const double dist(max(0.0, d - m_clear)); // fogged distance

    if (!col.is_unset())
      {
	Color fog(m_color);
	return fog.blend(col, exp(-dist*m_dens));
      }

    else
      return Neutral();
  }
} /* end of namespace */
