/*
 *  Gyro.cc -- ePiX2::Gyro (conformal motion) class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 09, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifdef DEBUG
#include "Debug.h"
#endif

#include "Functions.h"
#include "Point.h"
#include "Vector.h"
#include "Basis.h"

#include "Edge_Data.h"

#include "Gyro.h"

namespace ePiX2 {

  /*
  Scale_Factor::Scale_Factor(const double c) : m_scale(c) { }

  Scale_Factor& Scale_Factor::operator*= (const double c)
  {
    m_scale *= c;
    return *this;
  }

  double Scale_Factor::value(void) const
  {
    return m_scale;
  }
  */


  /* * * Gyro functions * * */
  Gyro::Gyro(const Basis& b, const double scale)
    : m_orient(b), m_scale(scale) { }

  Gyro& Gyro::move_to(const Point arg)
  {
    m_orient.move_to(arg);
    return *this;
  }

  Gyro& Gyro::move_by(const Vector& dX)
  {   
    m_orient.move_to(m_orient.here()+dX);
    return *this;
  }

  Gyro& Gyro::scale (const double c)
  {
    m_scale *= c;
    return *this;
  }

  Gyro& Gyro::reflect(const Vector& axis)
  {
    m_orient.reflect(axis);
    return *this;
  }

  Gyro& Gyro::rotate(const double angle, const Vector& axis)
  {
    m_orient.rotate(angle, axis);
    return *this;
  }

  Gyro& Gyro::rotate(const Vector& axis, const double angle)
  {
    m_orient.rotate(angle, axis);
    return *this;
  }


  bool Gyro::operator== (const Gyro& g) const
  {
    return ((m_orient == g.m_orient) && (m_scale == g.m_scale));
  }

  // express ourself in new coordinates
  Gyro& Gyro::in_coords(const Gyro& g)
  {
    Point new_O(g.in_coords(m_orient.here()));
    Vector e1(g.in_coords(m_orient.sea()));
    Vector e2(g.in_coords(m_orient.sky()));
    Vector e3(g.in_coords(m_orient.eye()));

    m_scale *= g.m_scale;
    m_orient = Basis(new_O, e1, e2, e3);
    return *this;
  }


  Point Gyro::here() const
  {
    return m_orient.here();
  }

  // convert Cartesian to our coordinates
  Point Gyro::in_coords(const Point& arg) const
  {
    return m_orient.coords(arg, m_scale);
  }

  Point Gyro::anti_coords(const Point& arg) const
  {
    return m_orient.anti_coords(arg, m_scale);
  }

  Vector Gyro::in_coords(const Vector& N) const
  {
    return in_coords(N.head()) - in_coords(N.tail());
  }

  Vector Gyro::anti_coords(const Vector& N) const
  {
    return m_orient.anti_coords(N.head(), m_scale) -
      m_orient.anti_coords(N.tail(), m_scale);
  }

  Edge3d Gyro::in_coords(const Edge3d& E) const
  {
    return Edge3d(in_coords(E.tail()), in_coords(E.head()), E.seen());
  }
} /* end of namespace */
