/*
 *  Gyro.h -- ePiX2 class for Euclidean motions and scaling
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 09, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                   *
 * A Gyro contains an orthonormal basis and a scale factor. An  *
 * Object can implement Euclidean motions or homotheties by applying *
 * transformations to its gyroscope rather than transforming a large *
 * internal set of data. The bookkeeping works because there is a    *
 * unique conformal motion carrying any one gyroscope to another.    *
 *                                                                   *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef EPIX2_GYROSCOPE
#define EPIX2_GYROSCOPE

#include "Basis.h"

namespace ePiX2 {

  class Point;
  class Vector;
  class Edge3d;

  /*
  // homothety factor that initializes to 1
  class Scale_Factor {
  public:
    Scale_Factor(const double c=1);
    Scale_Factor& operator*= (const double);
    double value(void) const;

  private:
    double m_scale;
  }; // end of class Scale_Factor
  */

  class Gyro {
  public:
    Gyro(const Basis& b=Basis(), const double scale=1);

    // next six functions re-defined by Compound
    Gyro& move_to(const Point arg); // set origin
    Gyro& move_by(const Vector&);   // translate

    Gyro& scale (const double);

    Gyro& reflect(const Vector& axis);
    Gyro&  rotate(const double angle, const Vector& axis);
    Gyro&  rotate(const Vector& axis, const double angle);

    bool operator== (const Gyro&) const;

    Point  here() const;

    // express ourselves in new coords
    Gyro& in_coords(const Gyro&);

    // express arg w.r.t. basis/scale
    Point  in_coords(const  Point& arg) const;
    Point  anti_coords(const  Point& arg) const;

    Vector in_coords(const Vector& arg) const;
    Edge3d in_coords(const Edge3d& arg) const;

    // Vector that converts to arg in our basis/scale
    Vector anti_coords(const Vector& arg) const;

  private:
    Basis m_orient;
    double m_scale;
  }; // end of Gyro

} /* end of namespace */

#endif /* EPIX2_GYROSCOPE */
