/*** 
 *** Interval.cc -- ePiX2::Interval class functions
 ***
 *** This file is part of ePiX, a program for creating high-quality 
 *** figures in LaTeX 
 ***
 *** Version 2.0pre
 *** Last Change: June 30, 2006
 ***
 *** 
 *** Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 *** Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 *** Department of Mathematics and Computer Science
 *** College of the Holy Cross
 *** Worcester, MA, 01610-2395, USA
 ***
 ***
 *** ePiX is free software; you can redistribute it and/or modify it
 *** under the terms of the GNU General Public License as published by
 *** the Free Software Foundation; either version 2 of the License, or
 *** (at your option) any later version.
 ***
 *** ePiX is distributed in the hope that it will be useful, but WITHOUT
 *** ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *** or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *** License for more details.
 ***
 *** You should have received a copy of the GNU General Public License
 *** along with ePiX; if not, write to the Free Software Foundation, Inc.,
 *** 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ***
 ***/
#include "Constants.h"
#include "Functions.h"
// #include "Errors.h"
#include "Interval.h"

namespace ePiX2 {

  Interval::Interval(const double a, const double b)
    : m_min(ePiX2::min(a, b)), m_max(ePiX2::max(a, b)) { }

  Interval Interval::literal(const double a, const double b)
  {
    Interval value(a, a);
    value.m_max = b;
    return value;
  }

  Interval Interval::Empty = literal(1,0);

  // Minkowski sum
  Interval& Interval::operator +=(const Interval& I)
    {
      if (m_max < m_min)
	; // we're empty, return below

      else if (I.m_max < I.m_min)
	*this = I;

      else
	{
	  m_min += I.m_min;
	  m_max += I.m_max;
	}

      return *this;
    }

  // intersection
  Interval& Interval::operator *=(const Interval& I)
  {
    if (m_min <= m_max) // we're non-empty
      {
	m_min = ePiX2::max(m_min, I.m_min);
	m_max = ePiX2::min(m_max, I.m_max);
      }

    return *this;
  }

  bool Interval::operator== (const Interval& I) const
  {
    if (m_max < m_min) // we're empty
      return (I.m_max < I.m_min); // I also empty
    else
      return ((m_min == I.m_min) && (m_max == I.m_max));
  }

  Interval operator+ (Interval I1, const Interval& I2)
  {
    return I1 += I2;
  }

  Interval operator* (Interval I1, const Interval& I2)
  {
    return I1 *= I2;
  }

} /* end of namespace */
