/*** 
 *** Interval.h -- ePiX2::Interval class and operations
 ***
 *** This file is part of ePiX, a program for creating high-quality 
 *** figures in LaTeX 
 ***
 *** Version 2.0pre
 *** Last Change: June 23, 2006
 ***
 *** 
 *** Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 *** Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 *** Department of Mathematics and Computer Science
 *** College of the Holy Cross
 *** Worcester, MA, 01610-2395, USA
 ***
 ***
 *** ePiX is free software; you can redistribute it and/or modify it
 *** under the terms of the GNU General Public License as published by
 *** the Free Software Foundation; either version 2 of the License, or
 *** (at your option) any later version.
 ***
 *** ePiX is distributed in the hope that it will be useful, but WITHOUT
 *** ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *** or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *** License for more details.
 ***
 *** You should have received a copy of the GNU General Public License
 *** along with ePiX; if not, write to the Free Software Foundation, Inc.,
 *** 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ***
 *** 
 *** This file provides the Interval class:
 ***
 ***     - Minkowski sum, intersection
 ***/

#ifndef EPIX2_INTERVAL
#define EPIX2_INTERVAL

namespace ePiX2 {

  class Interval {
  public:
    Interval(const double, const double);

    static Interval Empty;

    double min(void) const { return m_min; }
    double max(void) const { return m_max; }
    double avg(void) const { return 0.5*(m_min+m_max); }

    // Minkowski sum
    Interval& operator +=(const Interval&);

    // intersection
    Interval& operator *=(const Interval&);

    bool operator== (const Interval&) const;

    bool operator!= (const Interval& I) const
      { return !((*this) == I); }

    bool empty(void) const { return (m_max < m_min); }

  private:
    double m_min, m_max;

    static Interval literal(double a, double b); // can force b<a
  }; // end of Interval class

  Interval operator+ (Interval, const Interval&);
  Interval operator* (Interval, const Interval&);

} /* end of namespace */

#endif /* EPIX2_INTERVAL */
