/*
 *  Knife.cc -- ePiX2::Knife class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 12, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <string>

#include "Point.h"
#include "Basis.h"

#include "Gyro.h"

#include "Knife.h"

namespace ePiX2 {

  Knife& Knife::operator+= (const Vector& dX)
  {
    m_blade += dX;
    return *this;
  }
  Knife& Knife::move_to(const Point& arg)
  {
    m_blade.move_to(arg);
    return *this;
  }

  Knife& Knife::reflect(const Vector& axis)
  {
    m_blade.reflect_by(axis);
    return *this;
  }
  Knife& Knife::rotate(const double angle, const Vector& axis)
  {
    m_blade.rotate_by(angle, axis);
    return *this;
  }

  Knife Knife::anti_coords(const Gyro& gyro) const
  {
    return Knife(gyro.anti_coords(m_blade), m_pen);
  }

  /*
  // set color and width
  Knife& Knife::apply(const Ink& p)
  {
    m_pen.apply(p);
    return *this;
  }

  Knife& Knife::apply(const Pen& b)
  {
    m_pen.apply(b);
    return *this;
  }

  Knife& Knife::width(const std::string& len)
  {
    m_pen.width(len);
    return *this;
  }
  */

} /* end of namespace */
