/*
 *  Knife.h -- ePiX2::Knife class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 04, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                   *
 * A Knife chops objects, removing the portion that lies outside a   *
 * half-space, and possibly setting the color of any cut edges.      *
 * Users only need to set a Knife's attributes. Internal classes get *
 * access to data for actual chopping.                               *
 *                                                                   *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef EPIX2_KNIFE
#define EPIX2_KNIFE

#include <string>

#include "Vector.h"
#include "Styles.h"

namespace ePiX2 {

  class Point;
  class Gyro;

  class Knife {
  public:
    explicit Knife(const Vector& N, const Pen& pen = Pen::Xfine)
      : m_blade(N), m_pen(pen) { }

    Knife& operator+= (const Vector&); // translate
    Knife& move_to(const Point& arg);  // set origin

    Knife& reflect(const Vector& axis);
    Knife&  rotate(const double angle, const Vector& axis);

    // express us for chopping in given coordinates
    Knife anti_coords(const Gyro&) const;

    /*
    // set color and width
    Knife& apply(const Ink&);
    Knife& apply(const Pen&);
    Knife& width(const std::string&);
    */

  private:
    friend class World;
    friend class Plate3d;
    friend class Outline3d;
    friend class Wire3d;
    friend class Glyph3d;

    Vector m_blade;
    Pen m_pen;
  }; // end of class Knife

} /* end of namespace */

#endif /* EPIX2_LENGTH */
