/* 
 *  Layer.h -- ePiX2 binary space partition tree structure
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX
 *
 * Version 2.0pre
 * Last Change: July 30, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * This file provides:
 *
 *   Layer, a binary space partition structure for sorting Shards
 *   according to visibility. A Layer can be "photographed" in the
 *   following styles:
 *
 *   - photo:  Interiors shaded according to face normal
 *   - flash:  Interiors colored solid
 *   - trace:  Edges only, with hiding
 *   - x_ray:  Edges only, no hiding
 */

#ifndef EPIX2_LAYER
#define EPIX2_LAYER

#include <vector>

#include "Constants.h"

#include "Vector.h"

namespace ePiX2 {
  class Color;
  class Fog;

  class Camera;
  class Light;

  class Shard;
  class Screen;

  class Layer {
  public:
    // Layer uninitialized iff m_fragments.size() == 0
    explicit Layer(const Vector& N);
    ~Layer(void);

    void perp(const Vector& N);
    Vector perp(void) const;

    void clear(void);

    void add(const Shard*);   // insert into tree
    void add_f(const Shard*); // check for null and insert into tree
    void add_b(const Shard*);

    void add(const std::vector<Shard*>&);   // check each for need to split

    void shoot(Screen&, const Camera&, const Color& ambient,
	       const std::vector<Light>& spots, const Fog&,
	       const photo_type) const;

  private:
    Vector m_perp;
    std::vector<Shard*> m_fragments;

    Layer* front;
    Layer* back;
  }; // end of Layer

} /* end of namespace */

#endif /* EPIX2_LAYER */
