/*
 *  Lighting.h -- ePiX2::Light class and utility functions
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 14, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef EPIX2_LIGHT
#define EPIX2_LIGHT

#include <string>

#include "Point.h"
#include "Color.h"

namespace ePiX2 {

  class Fog;
  class Ink;
  class Vector;

  class Light {
  public:
    explicit Light(const Point& loc, const Color& tint=White())
      : m_loc(loc), m_tint(tint) { }

    double distance(const Point& arg) const;

    Point  here(void) const { return m_loc; }
    Color color(void) const { return m_tint; }

    // Color we cast at arg, seen through fog
    Color incident(const Fog& fog, const Point& arg) const;

  private:
    Point m_loc;
    Color m_tint;
  }; // end of class Light

  // Functions that model reflection from a line or surface element
  Color face_spot(const Ink& ink,
		  const Vector& perp,
		  const Vector& in_dir,  // *unit* vectors
		  const Vector& out_dir,
		  const Color& spot);

  Color face_ambient(const Ink& ink,
		     const Vector& perp,
		     const Vector& out_dir,
		     const Color& spot);

  /*
  Color line_spot(const Ink& ink,
		  const Vector& tgt,
		  const Vector& in_dir,
		  const Vector& out_dir,
		  const Color& spot);
  */

  Color line_ambient(const Ink& ink,
		     const Vector& tgt,
		     const Vector& out_dir,
		     const Color& spot);

} /* end of namespace */

#endif /* EPIX2_LIGHT */
