/* 
 *  Point.cc -- ePiX2::Point class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: September 19, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "Errors.h"
#include "Constants.h"

#include "Point.h"
#include "Vector.h"

namespace ePiX2 {

  Point::Point(double x1, double x2, double x3) 
    : Px1(x1), Px2(x2), Px3(x3) { }

  // Cartesian coordinate functions
  double Point::x1() const
  {
    return Px1;
  }
  double Point::x2() const
  {
    return Px2;
  }
  double Point::x3() const
  {
    return Px3;
  }

  // (in)equality (uses oo norm)
  bool Point::operator == (const Point& arg) const
  {
    return (fabs(Px1 - arg.Px1) < EPIX2_EPSILON &&
	    fabs(Px2 - arg.Px2) < EPIX2_EPSILON &&
	    fabs(Px3 - arg.Px3) < EPIX2_EPSILON);
  }

  bool Point::operator != (const Point& arg) const
  {
    return !((*this) == arg);
  }

  Point midpoint(const Point& arg1, const Point& arg2, const double t)
  {
    return Point(arg1.x1() + t*(arg2.x1()-arg1.x1()),
		 arg1.x2() + t*(arg2.x2()-arg1.x2()),
		 arg1.x3() + t*(arg2.x3()-arg1.x3()));			
  }
} /* end of namespace */
