/*
 *  SVG.h -- epix2::SVG output format
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: September 5, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef EPIX2_SVG_1_1
#define EPIX2_SVG_1_1

#include "Output.h"

namespace ePiX2 {

  class SVG : public Format {
  public:
    SVG(void);

    // location and offset loc; label string; text, fill colors; edge; padding;
    // alignment: font: rotation angle
    std::string print_text(const Pair&, const Pair&,
			   const std::string&, const mark_type,
			   const Color&, const Color&,
			   const Length&, const Pen2d&,
			   const align_type, const font_type,
			   const double) const;

    /*
    // location; marker; text, fill, edge colors; padding, border width;
    std::string print_mark(const Pair&, const mark_type, const Color&,
			   const Color&, const Pen2d&, const Length&) const;
    */

    // fill color; border; clipped flag; edges
    std::string print_plate(const Color&, const Pen2d&, bool,
			    const std::vector<Edge2d>&) const;

    // line/base; edge(s)
    std::string print_outline(const Pen2d&, const Pen2d&, 
			      const std::vector<Edge2d>&) const;

    std::string print_outline(const Pen2d&, const Pen2d&, 
			      const Edge2d&) const;


    // comment and file information block
    std::string file_header(void) const;

    std::string pic_header(const Pair& sw, const Pair& ne, const Length&,
			   const std::set<Color>&) const;

    std::string pic_footer(void) const;

    std::string print_color(const std::string&, const std::string&,
			    const double, const double, const double) const;

    std::string print_color(const std::string&, const std::string&,
			    const double, const double,
			    const double, const double) const;

    std::string fig_header(const Pair& sw, const Pair& ne, const Length&,
			   const std::set<Color>&) const;

    std::string fig_footer(void) const;

    /*
    std::string fig_caption(const std::string&) const;
    std::string fig_xref(const std::string&) const;
    */

    std::string comment(const std::string&) const;

  private:
    // no state data
    std::string update_fill_style(const Color&,
				  const Color&, const Length&) const;
    std::string update_line_style(const Color&,
				  const Length&, const Color&) const;

    std::string draw_outline_data(const std::vector<Edge2d>& edges,
				  const Pen2d& pen) const;

    std::string print(const Pair&) const;
    std::string print(const Color&) const;
    std::string print(const Length&) const;

    std::string print(const align_type arg) const;
    std::string print(const mark_type  arg) const;
    std::string print(const font_type  arg) const;

  }; // end of class SVG
} // end of namespace

#endif /* EPIX2_SVG_1_1 */
