/*
 *  Scissors.cc -- ePiX2::Scissors class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 12, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <cmath>

#include "Constants.h"

#include "Pair.h"
#include "Edge_Data.h"

#include "Scissors.h"

namespace ePiX2 {

  Scissors::Scissors(const Pair& tail, const Pair& head)
    : m_a(tail.x2() - head.x2()), m_b(head.x1() - tail.x1())
  {
    double denom(hypot(m_a, m_b));

    if (denom > EPIX2_EPSILON)
      {
	m_a /= denom;
	m_b /= denom;
      }

    m_c = -(m_a*tail.x1() + m_b*tail.x2());
  }

  double Scissors::height(const Pair& arg) const
  {
    return m_a*arg.x1() + m_b*arg.x2() + m_c;
  }

    // Location where segment crosses our blade
  Pair Scissors::cuts(const Edge2d& E) const
  {
    double h1(height(E.tail())), h2(height(E.head()));
    return (1.0/(h1-h2))*(h1*E.head() - h2*E.tail());   
  }

} /* end of namespace */
