/***
 ***  Scissors.h -- ePiX2::Scissors class
 ***
 *** This file is part of ePiX, a preprocessor for creating high-quality 
 *** line figures in LaTeX 
 ***
 *** Version 2.0pre
 *** Last Change: July 25, 2006
 ***
 *** 
 *** Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 *** Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 *** Department of Mathematics and Computer Science
 *** College of the Holy Cross
 *** Worcester, MA, 01610-2395, USA
 ***
 ***
 *** ePiX is free software; you can redistribute it and/or modify it
 *** under the terms of the GNU General Public License as published by
 *** the Free Software Foundation; either version 2 of the License, or
 *** (at your option) any later version.
 ***
 *** ePiX is distributed in the hope that it will be useful, but WITHOUT
 *** ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *** or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *** License for more details.
 ***
 *** You should have received a copy of the GNU General Public License
 *** along with ePiX; if not, write to the Free Software Foundation, Inc.,
 *** 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ***/

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                   *
 * A Scissors crops tiles, removing the portion that lies outside a  *
 * half-plane. Scissors are used privately by Screens to implement   *
 * cropping.                                                         *
 *                                                                   *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef EPIX2_SCISSORS
#define EPIX2_SCISSORS

#include <vector>

namespace ePiX2 {

  class Pair;
  class Edge2d;

  class Scissors {
  public:
    // defines the half-plane to the left of an oriented segment
    Scissors(const Pair&, const Pair&);

    // height(x,y) = ax + by + c = 0 at "blade"
    double height(const Pair&) const;

    // Location where segment crosses our blade
    Pair cuts(const Edge2d&) const;

  private:
    double m_a, m_b, m_c;
  }; // end of class Scissors

} /* end of namespace */

#endif /* EPIX2_LENGTH */
