/* 
 * Screen.h -- ePiX2::Screen class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 21, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * 
 * This file provides the:
 *   - Screen class, a maskable window
 */

#ifndef EPIX2_SCREEN
#define EPIX2_SCREEN

#include <set>
#include <vector>
#include <string>

#include "Mask.h"

namespace ePiX2 {

  /*  // Underlayer
  class Matte {
  public:

  private:
    Pen2d m_pen; // border
    Ink2d m_ink; // background
    std::string m_img; // image file
  }; // end of class Matte
  */

  class Pair;
  class Pair_Map;
  class Color;
  class Tile;
  class Plate2d;
  class Outline2d;

  class Screen {
    friend class Picture;
  public:
    // Screen(void);

    Screen(const Pair arg1, const Pair arg2);
    Screen(const Mask::Mask&);

    Screen(const Screen&);
    Screen& operator= (const Screen&);
    ~Screen();

    // superimpose a layer by affine scaling
    Screen& paste(const Screen& child);
    Screen& paste(const Screen& child, const Mask::Rect& inset);

    // or by a specified Pair_Map
    Screen& paste(const Screen& child, const Pair_Map&);

    Screen& clear(); // blank the screen
    Screen& crop();  // cull elements outside border
    Screen& crop_to(const Mask::Mask&);

    // decorations
    Screen& border(const Color&, std::string);
    Screen& border(const Pen&);
    Screen& backing(const Color&);

    // add data
    Screen& operator<< (const Color&);
    Screen& operator<< (const Tile&);

  private:
    Mask::Mask* m_mask;
    Outline2d* m_border;
    Plate2d* m_backing;

    std::set<Color> m_pallet;
    std::vector<Tile*> m_tiles;
  }; // end of Screen class

} /* end of namespace */

#endif /* EPIX2_SCREEN */
