/* 
 * Segment.cc -- ePiX2::Segment class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 20, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef DEBUG
#include "Debug.h"
#endif

#include "Constants.h"
#include "Functions.h"
#include "Point.h"
#include "Vector.h"

// Uncomment as necessary
// #include "Shard_0d.h"
#include "Shard_1d.h"
// #include "Shard_2d.h"

#include "Segment.h"

namespace ePiX2 {
  Segment::Segment(const Point& tail, const Point& head)
    : m_tail(tail), m_head(head)
  {
    // construct normal vector
    Vector N((head-tail)*E_3(tail));

    if (norm(N) < EPIX2_EPSILON) // nearly colinear with E_3
      N = E_2(tail);
    else
      N.normalize();

    Outline3d data(N);
    data.add(Edge3d(tail, head, true));
    data.update();
    add_shard(data);
  }

  Segment::Segment(const Vector& dir)
  {
    Segment(dir.tail(), dir.head());
  }

  // If Segment contains data, must define copy_ctor, operator=
  Segment::Segment(const Segment& obj)
    : Object(obj), m_tail(obj.m_tail), m_head(obj.m_head)
  { }

  Segment& Segment::operator=(const Segment& obj)
  {
    if (!(*this == obj))
      {
	// allocate any heap data, free resources, do whatever else...
	Object::operator=(obj);
	m_tail = obj.m_tail;
	m_head = obj.m_head;
      }

    return *this;
  }

  Segment* Segment::clone(void) const
  {
    return new Segment(*this);
  }

  bool Segment::operator== (const Segment& seg) const
  {
    return (m_tail == seg.m_tail && m_head == seg.m_head) ||
      (m_tail == seg.m_head && m_head == seg.m_tail);
  }

} // end of namespace

