/* 
 * Tetra.cc -- ePiX2::Tetrahedron class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 16, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef DEBUG
#include "Debug.h"
#endif

#include "Functions.h"
#include "Point.h"
#include "Vector.h"

// Uncomment as necessary
// #include "Shard_0d.h"
// #include "Shard_1d.h"
#include "Shard_2d.h"

#include "Tetra.h"

namespace ePiX2 {

  Vector median_perp(const Point& v1, const Point& v2, const Point& v3)
  {
    Point ctr(v1 + (1.0/3)*((v2-v1) + (v3-v1)));
    return ((v1-ctr)*(v2-ctr)).normalize();
  }

  Tetra::Tetra(const Point& v1, const Point& v2,
	       const Point& v3, const Point& v4)
  {
    // TO DO: Check for linear independence, use segment(s) if appropriate?
    Plate3d face1(median_perp(v2,v3,v4));

    face1.add(Edge3d(v2, v3, true));
    face1.add(Edge3d(v3, v4, true));
    face1.add(Edge3d(v4, v2, true));
    face1.update();

    Plate3d face2(median_perp(v1,v4,v3));

    face2.add(Edge3d(v1, v4, true));
    face2.add(Edge3d(v4, v3, true));
    face2.add(Edge3d(v3, v1, true));
    face2.update();

    Plate3d face3(median_perp(v1,v2,v4));

    face3.add(Edge3d(v1, v2, true));
    face3.add(Edge3d(v2, v4, true));
    face3.add(Edge3d(v4, v1, true));
    face3.update();

    Plate3d face4(median_perp(v2,v1,v3));

    face4.add(Edge3d(v2, v1, true));
    face4.add(Edge3d(v1, v3, true));
    face4.add(Edge3d(v3, v2, true));
    face4.update();

    add_shard(face1);
    add_shard(face2);
    add_shard(face3);
    add_shard(face4);
  }


  Tetra* Tetra::clone(void) const
  {
    return new Tetra(*this);
  }

  Tetra& Tetra::paint_face(const int i, const Ink& ink)
  {
    if (1 <= i && i <= 4)
      {
	fragment(i-1)->front_ink(ink);
	fragment(i-1)->back_ink(ink);
      }
    return *this;
  }

  Tetra& Tetra::limn_face(const int i, const Pen& pen)
  {
    if (1 <= i && i <= 4)
      {
	fragment(i-1)->front_pen(pen);
	fragment(i-1)->back_pen(pen);
      }
    return *this;
  }
} // end of namespace

