/* 
 *  Tile.h -- ePiX2 Screen Tile base class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 20, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * This file provides:
 *
 * Tile   - The base for screen elements (Plate, Outline, Glyph)
 */
#ifndef EPIX2_TILE
#define EPIX2_TILE

#include <string>

namespace ePiX2 {
  // Every drawable element in the Screen is a Tile
  class Pair_Map;
  class Scissors;
  class Mask::Mask;

  class Format;
  class Screen;

  class Tile {
  public:
    virtual ~Tile(void) { }

    virtual Tile* clone(void) const = 0;
    virtual Tile& map_by(const Pair_Map&) = 0;
    virtual Tile& crop_by(const Scissors&) = 0;
    virtual Tile& crop_to(const Mask::Mask&) = 0;

    virtual bool is_empty(void) const = 0;

    // Text Tile calculates its offset location in given length units
    virtual void realign(const std::string& len) { }
    virtual std::string print_to(const Format&) const = 0;
    virtual void add_colors_to(Screen&) const = 0;
  }; // end of class Tile

} /* end of namespace */

#endif /* EPIX2_TILE */
