/* 
 * Tile_0d.cc -- ePiX2::Label and Marker classes
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 20, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifdef DEBUG
#include "Debug.h"
#endif

#include <sstream>

#include "Functions.h"
#include "Pair.h"
#include "Pair_Map.h"
#include "Scissors.h"
#include "Mask.h"

#include "Length.h"

#include "Styles.h"

#include "Screen.h"

#include "Output.h"

#include "Tile_0d.h"

namespace ePiX2 {

  Glyph2d::Glyph2d(const Pair& here, const Length& hoff, const Length& voff,
		   const std::string& label, const mark_type mark,
		   const Color& text, const Color& back,
		   const Length& pad, const Pen2d& edge,
		   const align_type align, const font_type font,
		   const double angle, const bool seen)
    : m_here(here), m_offset_here(here),
      m_hoff(hoff), m_voff(voff), m_label(label), m_mark(mark),
      m_text(text), m_back(back), m_pad(pad), m_edge(edge), m_align(align),
      m_font(font), m_angle(angle), m_seen(seen)
  {
#ifdef DEBUG
    std::cerr << "Glyph2d() " << label << " at ("
	      << here.x1() << "," << here.x2() << ")" << std::endl;
#endif
  }

  Glyph2d& Glyph2d::map_by(const Pair_Map& f)
  {
#ifdef DEBUG
    std::cerr << "Glyph2d::map_by()" << std::endl;
#endif
    m_here = f(m_here);
    return *this;
  }

  Glyph2d& Glyph2d::crop_by(const Scissors& S)
  {
#ifdef DEBUG
    std::cerr << "Glyph2d::crop_by()" << std::endl;
#endif
    if (S.height(m_here) < 0)
      m_seen=false;

    return *this;
  }

  Glyph2d& Glyph2d::crop_to(const Mask::Mask& M)
  {
#ifdef DEBUG
    std::cerr << "Glyph2d::crop_to()" << std::endl;
#endif
    M.crop(this);
    return *this;
  }

  Glyph2d* Glyph2d::clone(void) const
  {
#ifdef DEBUG
    std::cerr << "Glyph2d::clone()" << std::endl;
#endif
    return new Glyph2d(*this);
  }

  bool Glyph2d::is_empty(void) const
  {
    return !m_seen;
  }

  void Glyph2d::realign(const std::string& len)
  {
    m_offset_here = m_here + Pair(m_hoff.to(len).magnitude(),
				  m_voff.to(len).magnitude());
  }

  std::string Glyph2d::print_to(const Format& fmt) const
  {
#ifdef DEBUG
    std::cerr << "Glyph2d::print_to()" << std::endl;
#endif
    if (is_empty())
      return "";

    // else
    std::ostringstream obuf;

    obuf << fmt.print_text(m_here, m_offset_here, 
			   m_label, m_mark,
			   m_text, m_back, m_pad, m_edge,
			   m_align, m_font, m_angle);
    return obuf.str();
  }

  void Glyph2d::add_colors_to(Screen& S) const
  {
#ifdef DEBUG
    std::cerr << "Glyph2d::add_colors_to()" << std::endl;
#endif
    S << m_text;

    if (!m_back.is_unset())
      S << m_back;

    if (!m_edge.color().is_unset())
      S << m_edge.color();
  }

} // end of namespace

