/* 
 *  Compound.h -- ePiX2::Compound Object class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX
 *
 * Version 2.0pre
 * Last Change: August 16, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef EPIX2_COMPOUND
#define EPIX2_COMPOUND

#include "Object.h"

namespace ePiX2 {

  class Point;
  class Vector;
  class Gyro;

  class Knife;

  class Pen;
  class Ink;
  class Stencil;

  class Shard;

  class Compound : public Object {
  public:
    Compound();

    // class bookkeeping
    Compound(const Compound&);
    Compound& operator=(const Compound&);

    ~Compound();
    // bool operator== (const Compound&) const;

    Compound* clone() const;

    // add an Object
    Compound& operator<< (const Object&);

    // cuts
    Compound&  chop(const Knife&); // half-space cut
    Compound& slice(const Knife&); // planar cut

    // set attributes
    Compound& front_pen(const Pen&);
    Compound& front_ink(const Ink&);
    Compound& back_pen(const Pen&);
    Compound& back_ink(const Ink&);

    Compound& line_pen(const Pen&);
    Compound& base_pen(const Pen&);

    Compound& stencil(const Stencil&);
    Compound& text(const Color&);
    Compound& back(const Color&);
    Compound&  pad(const Length&);
    Compound& edge(const Pen&);

    Compound& align(const align_type);
    Compound&  font(const font_type);
    Compound& angle(const double);
    // "h_offset x v_offset", e.g., "2x2" (pt), "4x-2pt", "1mm x -4pt"
    Compound& offset(std::string);

    Compound& fix();


    // do we occlude X from vpt?
    bool hides(const Point& vpt, const Point& X) const;

    // add our shards to specified container
    void shatter(std::vector<Shard*>&);

  private:
    // data
    std::vector<Object*> m_atoms;

  }; // end of class Compound
  /*
  Compound Tetrahedron(const Point&, const Point&, const Point&, const Point&);

  // opposite corners, sides parallel to coord planes
  Compound Block(const Point&, const Point&);
  */
} // end of namespace

#endif /* EPIX2_COMPOUND */
