/* 
 * New_Object.h -- ePiX2::New_Object class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 10, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * Many Object-derived classes (e.g. Polygon) need only constructors that
 * represent them as a list of Shards. Most others contain a small amount
 * of extra geometric data (e.g. Circle). A very few (e.g. Compound) will
 * be substantially represented by data other than a std::vector of Shard
 * pointers. The guidelines below explain which Object functions need to 
 * be overridden in each case. Each list is cumulative, tacitly encompassing
 * the list(s) above it.
 *
 * (Rep'd by Shards only, contains no non-base data)
 *   [] Constructors
 *   [] clone
 *
 * (Rep'd by Shards and geometric data)
 *   [] Copy constructor, assignment (must explicitly call Object::versions)
 *
 * (Rep'd other than by Shard list, contain heap resources)
 *   [] Destructor
 *   [] chop, slice
 *   [] attribute-setting (front_pen et. al., see below)
 *   [] hides
 *   [] shatter
 */

#ifndef EPIX2_New_Object
#define EPIX2_New_Object

#include "Object.h"

namespace ePiX2 {

  class New_Object : public Object {
  public:
    New_Object(); // default constructor not absolutely necessary

    //    New_Object(const New_Object&);
    //    New_Object& operator=(const New_Object&);

    New_Object* clone(void) const;
    //    ~New_Object();

    //    bool operator== (const New_Object&) const;

    //    New_Object&  chop(const Knife&); // half-space cut
    //    New_Object& slice(const Knife&); // planar cut

    // set attributes
    /*
    New_Object& front_pen(const Pen&);
    New_Object& front_ink(const Ink&);
    New_Object& back_pen(const Pen&);
    New_Object& back_ink(const Ink&);

    // null functions
    New_Object& line_pen(const Pen&);
    New_Object& base_pen(const Pen&);
    New_Object& stencil(const Stencil&);

    New_Object& text(const Color&);
    New_Object& back(const Color&);
    New_Object&  pad(const Length&);
    New_Object& edge(const Pen&);

    New_Object& align(const align_type);
    New_Object&  font(const font_type);
    New_Object& angle(const double);
    // "h_offset x v_offset", e.g., "2x2" (pt), "4x-2pt", "1mm x -4pt"
    New_Object& offset(std::string);
    */

    // do we occlude X from vpt?
    //    bool hides(const Point& vpt, const Point& X) const;

    // add Shard(s) to specified container
    //    void shatter(std::vector<Shard*>&);

  private:
  }; // end of class New_Object

} /* end of namespace */

#endif /* EPIX2_New_Object */
