/*
 *  Output.h -- epix2::Output class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: September 5, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
/* This file provides:
 *
 *   The Format base class
 */

#ifndef EPIX2_OUTPUT
#define EPIX2_OUTPUT

#include <set>
#include <vector>
#include <string>

#include "Styles.h"

namespace ePiX2 {

  class Pair;
  class Edge2d;

  class Picture;

  class Format {
  public:
    virtual ~Format(void) { }

    /* * * Functions that write Elements * * */
    virtual std::string print_text(const Pair&, const Pair&,     // locations
				   const std::string&,           // label text
				   const mark_type,
				   const Color&, const Color&,   // text, back
				   const Length&,                // padding
				   const Pen2d&,                 // border
				   const align_type, const font_type,
				   const double) const = 0;      // angle

    /*
    // location; marker; text, fill colors; border; padding
    virtual std::string print_mark(const Pair&, 
				   const mark_type,
				   const Color&, const Color&,
				   const Pen2d&,
				   const Length&) const = 0;
    */

    // fill color; border; clipped; edges
    virtual std::string print_plate(const Color&, const Pen2d&, const bool,
				  const std::vector<Edge2d>&) const = 0;

    // line, base, edge(s)
    virtual std::string print_outline(const Pen2d&, const Pen2d&,
				      const std::vector<Edge2d>&) const = 0;

    virtual std::string print_outline(const Pen2d&, const Pen2d&,
				      const Edge2d&) const = 0;

    // line, base; edges
    //    virtual std::string print_wire(const Pen2d&, const Pen2d&, 
    //				   const std::vector<Edge2d>&) const = 0;

    // comment and file information block
    std::string file_header(void) const;

    // Corners, unitlength, pallet
    virtual std::string pic_header(const Pair& sw, const Pair& ne,
				   const Length&,
				   const std::set<Color>&) const = 0;

    virtual std::string pic_footer(void) const = 0;

    // model, name, densities
    virtual std::string print_color(const std::string&, const std::string&,
				    const double, const double,
				    const double) const = 0;

    virtual std::string print_color(const std::string&, const std::string&,
				    const double, const double,
				    const double, const double) const = 0;


    // e.g., local description
    /*
    virtual std::string fig_header(const Pair& sw, const Pair& ne,
				   const Length&,
				   const std::set<Color>&) const = 0;

    virtual std::string fig_footer(void) const = 0;

    virtual std::string fig_caption(const std::string&) const = 0;
    // e.g., cross-referencing tag, URL
    virtual std::string fig_xref(const std::string&) const = 0;
    */

    // output file comment
    virtual std::string comment(const std::string&) const = 0;

  protected:

    // Low-level chunks
    virtual std::string print(const Pair&) const = 0;
    virtual std::string print(const Color&) const = 0;
    virtual std::string print(const Length&) const = 0;

    virtual std::string print(const align_type arg) const = 0;
    virtual std::string print(const mark_type arg) const = 0;
    virtual std::string print(const font_type arg) const = 0;

  }; // end of class Format

  // Global function passing non-constant Picture ref
  void print(Picture&, const char*, const Format&);

} /* end of namespace */

#endif /* EPIX2_OUTPUT */
