/* 
 *  Picture.h -- epix2::Picture class
 *
 * This file is part of ePiX2, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 23, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                   *
 * ePiX2 represents figures in a rectangular markup region, Picture, *
 * which has true dimensions, a border and background, and cropping  *
 * capability.                                                       *
 *                                                                   *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef EPIX2_PICTURE
#define EPIX2_PICTURE

#include <string>

#include "Length.h"
#include "Screen.h"

namespace ePiX2 {
  class Pair;
  class Color;
  class Pen;

  class Format;

  class Picture {
  public:
    // corners and dimensions, e.g. "4in x 15cm" or "4 x 6in"
    Picture(const Pair arg1, const Pair arg2, std::string);

    // same syntax as size
    Picture& offset(const std::string&);

    // Pass-through functions
    // superimpose a layer by affine scaling
    Picture& paste(const Screen& child);
    // place child (or its cutout) into inset
    Picture& paste(const Screen& child, const Mask::Rect& inset);
    //Picture& paste(const Screen& child, const Rect& cutout, const Rect& inset);

    // Pair_Map specifies inset
    Picture& paste(const Screen& child, const Pair_Map&);
    //Picture& paste(const Screen& child, const Rect& cutout, const Pair_Map&);

    // edit contents
    Picture& clear(void);          // empty the screen
    Picture& crop(void);
    Picture& crop_to(const Mask::Rect&); // cull elements outside mask

    // change decorations
    Picture& border(const Color&, std::string);
    Picture& border(const Pen&);
    Picture& backing(const Color&);


    // Internal functions
    // cache data
    Picture& operator<< (const Color&);
    Picture& operator<< (const Tile&);

    // Let others write to m_data
    Screen& screen(void);

    void print_to(const char* filename, const Format& fmt);

  private:
    Screen m_data;
    Screen m_page; // one corner at (0,0)

    Length m_width;
    Length m_height;

    Length m_hoff;
    Length m_voff;
  }; // end of class Picture

} /* end of namespace */

#endif /* EPIX2_PICTURE */
