/* 
 * Segment.h -- ePiX2::Segment class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX 
 *
 * Version 2.0pre
 * Last Change: August 20, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * Many Object-derived classes (e.g. Polygon) need only constructors that
 * represent them as a list of Shards. Most others contain a small amount
 * of extra geometric data (e.g. Circle). A very few (e.g. Compound) will
 * be substantially represented by data other than a std::vector of Shard
 * pointers. The guidelines below explain which Object functions need to 
 * be overridden in each case. Each list is cumulative, tacitly encompassing
 * the list(s) above it.
 *
 * (Rep'd by Shards only, contains no non-base data)
 *   [] Constructors
 *   [] clone
 *
 * (Rep'd by Shards and geometric data)
 *   [] Copy constructor, assignment (must explicitly call Object::versions)
 *
 * (Rep'd other than by Shard list, contain heap resources)
 *   [] Destructor
 *   [] chop, slice
 *   [] attribute-setting (front_pen et. al., see below)
 *   [] hides
 *   [] shatter
 */

#ifndef EPIX2_SEGMENT
#define EPIX2_SEGMENT

#include "Point.h"
#include "Object.h"

namespace ePiX2 {

  class Vector;
  class Segment : public Object {
  public:
    Segment(const Point&, const Point&);
    explicit Segment(const Vector&);

    Segment(const Segment&);
    Segment& operator=(const Segment&);

    Segment* clone(void) const;

    bool operator== (const Segment&) const;

  private:
    Point m_tail;
    Point m_head;
  }; // end of class Segment

} /* end of namespace */

#endif /* EPIX2_SEGMENT */
