/* 
 * Shard_0d.h -- ePiX2::Label and Marker classes
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX
 *
 * Version 2.0pre
 * Last Change: August 16, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef EPIX2_SHARD_0D
#define EPIX2_SHARD_0D

#include <vector>
#include <string>

#include "Vector.h"

#include "Length.h"

#include "Styles.h"
#include "Shard.h"

namespace ePiX2 {
  class Point;

  class Knife;
  class Layer;

  class Gyro;

  class Screen;

  class Fog;
  class Light;
  class Camera;

  class Glyph3d : public Shard {
  public:
    Glyph3d(const Vector& perp, const Length& hoff, const Length& voff,
	    const std::string& label, const mark_type mark,
	    const Stencil& stencil);

    Glyph3d* clone(void) const;

    // half-space cut; Xfine leaves cuts undrawn
    Glyph3d*  chop(const Knife&, std::vector<Shard*>&);

    // quasi-planar cut; no default Pen
    Glyph3d* slice(const Knife&, std::vector<Shard*>&);

    Glyph3d& stencil(const Stencil&);

    Glyph3d& text(const Color&);
    Glyph3d& back(const Color&);
    Glyph3d&  pad(const Length&);
    Glyph3d& edge(const Pen&);

    Glyph3d& align(const align_type);
    Glyph3d&  font(const font_type);
    Glyph3d& angle(const double);

    Glyph3d& offset(std::string);

    Glyph3d& fix();

    // photograph to specified Screen
    void shoot(Screen&, const Camera&, const Color& ambient,
	       const std::vector<Light>& no_op, const Fog&,
	       photo_type) const;

    void update(void) const { }

    // Returns (#edges)*EPIX2_EPSILON
    double priority(void) const { return 0; }

    bool is_empty(void) const;
    Glyph3d* in_coords(const Gyro&);

    // place in BSP
    void add_to(Layer*) const;

    bool hides(const Point& vpt, const Point& X) const { return false; }

    // Our location, for reflectivity calculation
    Point here(void) const;
    // Unit normal vector
    Vector perp(void) const;

  private:
    // m_perp can change when we're added to a Layer
    mutable Vector m_perp;

    Length m_hoff;
    Length m_voff;

    std::string m_label;
    mark_type m_mark;

    Stencil m_stencil;

    bool m_seen;
  }; // end of class Glyph3d

} /* end of namespace */

#endif /* EPIX2_SHARD_0D */
