/* 
 * Shard_1d.h -- epix2::Outline3d class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX
 *
 * Version 2.0pre
 * Last Change: August 16, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * This file provides the planar path classes Outline3d and Outline2d.
 */

#ifndef EPIX2_SHARD_1D
#define EPIX2_SHARD_1D

#include <vector>
#include <string>

#include "Constants.h"
#include "Vector.h"

#include "Edge_Data.h"

#include "Styles.h"
#include "Shard.h"

namespace ePiX2 {
  class Point;

  class Fog;
  class Light;
  class Layer;

  class Gyro;

  class Screen;
  class Format;
  class Camera;

  class Outline3d : public Shard {
  public:
    Outline3d(const Pen&, const Pen&, // line and base
	      bool clipped,           // all edges drawn?
	      const Vector&);         // perp vector

    Outline3d(const Vector&);

    ~Outline3d(void);

    Outline3d* clone(void) const;

    // half-space cut; Xfine leaves cuts undrawn
    Outline3d*  chop(const Knife&, std::vector<Shard*>&);

    // quasi-planar cut; no default Pen
    Outline3d* slice(const Knife&, std::vector<Shard*>&);

    Outline3d& line_pen(const Pen&);
    Outline3d& base_pen(const Pen&);

    Outline3d& fix();

    // photograph to specified Screen
    void shoot(Screen&, const Camera&, const Color& ambient,
	       const std::vector<Light>& no_op, const Fog&,
	       photo_type) const;

    void update(void) const; // update area

    // Returns (#edges)*EPIX2_EPSILON
    double priority(void) const;

    bool is_empty(void) const;
    Outline3d* in_coords(const Gyro&);

    // place in BSP
    void add_to(Layer*) const;

    bool hides(const Point& vpt, const Point& X) const { return false; }

    // Our location, for reflectivity calculation
    Point here(void) const;
    // Unit normal vector
    Vector perp(void) const;

    Outline3d& add(const Edge3d&);

  private:
    Pen m_line;
    Pen m_base;

    bool m_clipped; // some edge(s) undrawn

    // bookkeeping data
    Vector m_perp;          // unit normal; location = m_perp.tail()
    mutable double m_area;  // (#edges)*EPIX2_EPSILON
    mutable bool m_updated; // area current?

    std::vector<Edge3d> m_border;

  }; // end of class Outline3d

} /* end of namespace */

#endif /* EPIX2_SHARD_1D */
