/* 
 * Tile_0d.h -- ePiX2::Label and Marker classes
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX
 *
 * Version 2.0pre
 * Last Change: August 20, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef EPIX2_TILE_0D
#define EPIX2_TILE_0D

#include <vector>
#include <string>

#include "Pair.h"
#include "Length.h"
#include "Styles.h"
#include "Tile.h"

namespace ePiX2 {

  class Scissors;
  class Mask::Mask;
  class Pair_Map;

  class Edge2d;
  class Wire2d;

  class Screen;
  class Format;

  class Glyph2d : public Tile {
  public:
    Glyph2d(const Pair& here, const Length& hoff, const Length& voff,
	    const std::string& label, const mark_type mark,
	    const Color& text, const Color& back,
	    const Length& pad, const Pen2d& edge,
	    const align_type align, const font_type font,
	    const double angle, const bool seen);

    Glyph2d& map_by(const Pair_Map&);
    Glyph2d& crop_by(const Scissors&);
    Glyph2d& crop_to(const Mask::Mask&);

    Glyph2d* clone(void) const;

    bool is_empty(void) const;

    // required by interface
    void add(const Edge2d& E) { }
    void add(const Wire2d& E) { }

    void realign(const std::string&);
    std::string print_to(const Format&) const;
    void add_colors_to(Screen&) const;

  private:
    Pair m_here;
    Pair m_offset_here;

    Length m_hoff;
    Length m_voff;

    std::string m_label;
    mark_type m_mark;

    Color m_text;
    Color m_back;
    Length m_pad;
    Pen2d m_edge;
    align_type m_align;
    font_type  m_font;
    double m_angle;

    bool m_seen;
  }; // end of class Glyph2d

} /* end of namespace */

#endif /* EPIX2_TILE_0D */
