/* 
 * Tile_1d.h -- epix2::Outline2d class
 *
 * This file is part of ePiX, a program for creating high-quality 
 * figures in LaTeX
 *
 * Version 2.0pre
 * Last Change: August 20, 2006
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * This file provides the planar path classes Outline3d and Outline2d.
 */

#ifndef EPIX2_TILE_1D
#define EPIX2_TILE_1D

#include <vector>
#include <string>

#include "Edge_Data.h"

#include "Styles.h"
#include "Tile.h"

namespace ePiX2 {
  class Scissors;
  class Mask::Mask;
  class Pair_Map;

  class Screen;
  class Format;

  class Outline2d : public Tile {
  public:
    Outline2d(const Pen2d&, const Pen2d&, bool clipped);

    Outline2d& map_by(const Pair_Map&);
    Outline2d& crop_by(const Scissors&);
    Outline2d& crop_to(const Mask::Mask&);

    Outline2d* clone(void) const;

    void add(const Edge2d&);
    void add(const Wire2d&);
    bool is_empty(void) const;

    std::string print_to(const Format&) const;
    void add_colors_to(Screen&) const;

  private:
    Pen2d m_line;
    Pen2d m_base;

    bool m_clipped; // some edge(s) undrawn?

    std::vector<Edge2d> m_border;

  }; // end of class Outline2d

} // end of namespace

#endif /* EPIX2_TILE_1D */
